import { PluginObject } from 'vue'

declare namespace GeolocationPlugin {
  interface PositionOptions {
    enableHighAccuracy?: boolean
    timeout?: number
    maximumAge?: number
  }

  interface GeoJSONPoint {
    type: 'Point'
    coordinates: [number, number]
  }

  interface Position {
    readonly coords: {
      readonly latitude: number
      readonly altitude: number
      readonly longitude: number
      readonly accuracy: number
      readonly altitudeAccuracy: number
      readonly heading: number
      readonly speed: number
    }
    readonly timestamp: DOMTimeStamp
  }

  interface Options {
    watch: boolean
    options: PositionOptions
  }

  interface Geolocation {
    coords: Position.coords | null
    timestamp: Position.timestamp | null
    geoJSON: GeoJSONPoint | null
    loading: boolean
    supported: boolean

    getCurrentPosition: (options?: Options) => Promise<Position>
    setOption: (key: string, value: number | boolean) => void
    checkSupport: () => boolean
  }
}

declare module 'vue/types/vue' {
  interface Vue {
    readonly $geolocation: GeolocationPlugin.Geolocation
  }
}

declare module '@nuxt/types' {
  interface NuxtAppOptions {
    readonly $geolocation: GeolocationPlugin.Geolocation
  }
  interface Context {
    readonly $geolocation: GeolocationPlugin.Geolocation
  }
}

interface VueGeolocation extends PluginObject<undefined> {}

declare const VueGeolocation: VueGeolocation
export = VueGeolocation
